package com.surveytown;

import org.json.simple.JSONValue;
import org.json.simple.JSONObject;

import org.apache.http.NameValuePair;
import java.util.List;
import java.util.ArrayList;
import java.io.InputStream;
import java.io.BufferedReader;
import java.io.InputStreamReader;

import java.io.IOException;
import java.io.UnsupportedEncodingException;
import org.apache.http.client.*;
import org.apache.http.message.*;
import org.apache.http.client.methods.HttpPost;
import org.apache.http.impl.client.*;
import org.apache.http.HttpResponse;
import org.apache.http.HttpEntity;
import org.apache.commons.logging.*;
import org.apache.http.client.entity.UrlEncodedFormEntity;
import org.apache.http.protocol.HTTP;
import java.util.HashMap;


public class SurveyTown {
	protected String key;
	protected String url = "api.myfeedback.io/2/REST/";
	private String version = "2.5.0";
	
	public SurveyTown( String key, Boolean useHTTPS ) {
		String proto = "http://";
		
		this.key = key;
		if ( useHTTPS ) {
			proto = "https://";
		}
		
		this.url = proto+this.url;
	}
	
	public SurveyTown( String key ) {
		this( key, false );
	}
	
	                
	                
	/**
	 * Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE
	 *
	 * Please contact SurveyTown support for details, this function is only available to reseller accounts
	 *
	 	 * @param  clientId The client ID you are creating the cookie for, must be a client belonging to your reseller account
	 * @param String username The belonging to the client ID you are logging in with the cookie
	 * @param  time Cookie timeout in seconds
	 * @return Object Cookie value to be set in SURVEYS_SSO_COOKIE
	 */ 
	public Object Integration_Get_Cookie( int clientId, String username, int time ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Integration_Get_Cookie" );
		args = this.makeArg( args, "clientId", clientId );
		args = this.makeArg( args, "username", username );
		args = this.makeArg( args, "time", time );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE using your client's credentials
	 *
	 * Please contact SurveyTown support for details, this function is only available to reseller accounts
	 *
	 	 * @param String username The of the client
	 * @param String password The current of the client
	 * @param  time Cookie timeout in seconds
	 * @return Object Returns a struct on a valid username/password combo
	 */ 
	public Object Integration_Login_Get_Cookie( String username, String password, int time ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Integration_Login_Get_Cookie" );
		args = this.makeArg( args, "username", username );
		args = this.makeArg( args, "password", password );
		args = this.makeArg( args, "time", time );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Retrieve a survey response
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to retrieve the response from
	 * @param  responseId The ID of the response you want to retrieve
	 * @return Object Array of response properties
	 */ 
	public Object Response_Get( int surveyId, int responseId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Response_Get" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "responseId", responseId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Create a survey response
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to respond to
	 * @param HashMap<String, String> updateParameters A struct of update parameters, any combination of which can be used
	 * @param HashMap<String, String> optionalParameters A struct of optional parameters, see below for valid keys
	 * @return Object Returns responseId on success
	 */ 
	public Object Response_Create( int surveyId, HashMap<String, String> updateParameters, HashMap<String, String> optionalParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Response_Create" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "updateParameters", updateParameters );
		args = this.makeArg( args, "optionalParameters", optionalParameters );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Update an existing survey response
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to update the response of
	 * @param  responseId The ID of the response you want to update
	 * @param HashMap<String, String> updateParameters A struct of update parameters, any combination of which can be used
	 * @param HashMap<String, String> optionalParameters A struct of optional parameters, see below for valid keys
	 * @return Object Returns responseId on success
	 */ 
	public Object Response_Update( int surveyId, int responseId, HashMap<String, String> updateParameters, HashMap<String, String> optionalParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Response_Update" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "responseId", responseId );
		args = this.makeArg( args, "updateParameters", updateParameters );
		args = this.makeArg( args, "optionalParameters", optionalParameters );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Delete a response from a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to delete from
	 * @param  responseId The ID of the response you want to delete
	 * @return Object Returns true on success
	 */ 
	public Object Response_Delete( int surveyId, int responseId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Response_Delete" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "responseId", responseId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Start a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to start
	 * @return Object Returns true on success
	 */ 
	public Object Survey_Start( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Start" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Stop a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to stop
	 * @return Object Returns true on success
	 */ 
	public Object Survey_Stop( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Stop" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Copy a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to copy
	 * @return Object Returns The ID of the new survey
	 */ 
	public Object Survey_Copy( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Copy" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Delete a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to delete
	 * @return Object Returns true on success
	 */ 
	public Object Survey_Delete( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Delete" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Get a listing of surveys
	 *
	 * 
	 *
	 	 * @return Object Array of surveys with the key of surveyId
	 */ 
	public Object Survey_List(  ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_List" );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Find surveys based on specific criteria
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.
	 *
	 * 
	 * If the 'offset' optionalParameter is provided, the result set will start at that parameter's value (indexed from 0).
	 *
	 	 * @param HashMap<String, String> optionalParameters A struct of optional parameters, any combination of which can be used
	 * @return Object Array of surveys with the key of surveyId and values of name, title, status, start_date, and end_date, ordered by survey ID ascending
	 */ 
	public Object Survey_Find( HashMap<String, String> optionalParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Find" );
		args = this.makeArg( args, "optionalParameters", optionalParameters );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Retrieve a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to retrieve
	 * @return Object Survey properties including questions, logic & actions
	 */ 
	public Object Survey_Get_Info( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Get_Info" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Retrieves the questions for a particular survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want to retrieve the questions for
	 * @return Object Array of survey questions, with the key of the question ID. The response format is similar to that of Survey_Get_Responses.
	 */ 
	public Object Survey_Get_Questions( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Get_Questions" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Counts how many responses a survey has.
	 *
	 * 
	 *
	 	 * @param  surveyId ID of the survey you want to retrieve the response counts for
	 * @return Object Count of how many responses there are for the given survey
	 */ 
	public Object Survey_Get_Response_Count( int surveyId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Get_Response_Count" );
		args = this.makeArg( args, "surveyId", surveyId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Retrieve responses a specific contact has entered
	 *
	 * 
	 *
	 	 * @param String emailAddress The email address of the contact you want to retrieve responses for
	 * @param HashMap<String, String> optionalParameters A struct of optional parameters, any combination of which can be used
	 * @return Object Array of responses. See Survey_Get_Responses for response format documentation.
	 */ 
	public Object Survey_Get_Responses_By_Email( String emailAddress, HashMap<String, String> optionalParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Get_Responses_By_Email" );
		args = this.makeArg( args, "emailAddress", emailAddress );
		args = this.makeArg( args, "optionalParameters", optionalParameters );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Retrieve responses for a specific survey
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.
	 *
	 * 
	 * If the 'offset' optionalParameter is provided, the result set will start at that parameter's value (indexed from 0).
	 * 
	 * The return value is an array of responses. Each response has array keys for the question ID the response goes with.
	 * 
	 * The format for a simple textbox question's response looks like this example:
	 * 
	 * "24": {
	 *     "response": {
	 *         "choiceId": null,
	 *         "responseValue": "A Developer"
	 *     },
	 *     "questionText": "Who are you? - keep it to one line please.",
	 *     "questionId": 24,
	 *     "questionType": "singleText",
	 *     "token": null,
	 *     "isRequired": false
	 * }
	 * 
	 * Any question that can have only one response will have an array key named "response". Questions that can have more than one response value do /not/ have a "response" key; instead, they have a "responses" key, as seen in this example:
	 * 
	 * "29": {
	 *     "responses": [
	 *         {
	 *             "choiceId": 98,
	 *             "choiceText": "Blake",
	 *             "responseValue": 6
	 *         },
	 *         {
	 *             "choiceId": 99,
	 *             "choiceText": "Brian",
	 *             "responseValue": 5
	 *         },
	 *     ],
	 *     "questionText": "What number is each person thinking of? (remember, their numbers, and therefore also your guesses, must sum to 1001)",
	 *     "questionId": 29,
	 *     "questionType": "addSum",
	 *     "token": null,
	 *     "isRequired": false
	 * }
	 *
	 	 * @param  surveyId The ID of the survey you want to retrieve the questions for
	 * @param HashMap<String, String> optionalParameters A struct of optional parameters, any combination of which can be used
	 * @return Object Array of surveys
	 */ 
	public Object Survey_Get_Responses( int surveyId, HashMap<String, String> optionalParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Get_Responses" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "optionalParameters", optionalParameters );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Retrieve responses for a specific survey
	 *
	 * See documentation for Survey_Get_Responses for optionalParameters
	 *
	 	 */ 
	public Object Survey_Get_Responses_Zapier(  ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Get_Responses_Zapier" );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Add a tag to a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want the tag added to
	 * @param  tagId The ID of the tag you want to add
	 * @return Object Returns true on success
	 */ 
	public Object Survey_Add_Tag( int surveyId, int tagId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Add_Tag" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "tagId", tagId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Remove a tag from a survey
	 *
	 * 
	 *
	 	 * @param  surveyId The ID of the survey you want the tag removed from
	 * @param  tagId The ID of the tag you want to remove
	 * @return Object Returns true on success
	 */ 
	public Object Survey_Remove_Tag( int surveyId, int tagId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Survey_Remove_Tag" );
		args = this.makeArg( args, "surveyId", surveyId );
		args = this.makeArg( args, "tagId", tagId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Create a new tag
	 *
	 * 
	 *
	 	 * @param String name The of your tag
	 * @return Object id Returns the Tag ID of your new tag
	 */ 
	public Object Tag_Create( String name ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Tag_Create" );
		args = this.makeArg( args, "name", name );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Update an existing Tag
	 *
	 * Update Parameters can include:
	 *  - name: The name of your tag
	 *
	 	 * @param  tagId The of the tag you want to modify
	 * @param HashMap<String, String> updateParameters A struct of replacement values for your tag - only specify items that you want to change
	 * @return Object Returns true on success
	 */ 
	public Object Tag_Update( int tagId, HashMap<String, String> updateParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Tag_Update" );
		args = this.makeArg( args, "tagId", tagId );
		args = this.makeArg( args, "updateParameters", updateParameters );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Delete a tag you created
	 *
	 * 
	 *
	 	 * @param  tagId The Tag ID of the tag you wish to delete
	 * @return Object Returns true on success
	 */ 
	public Object Tag_Delete( int tagId ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Tag_Delete" );
		args = this.makeArg( args, "tagId", tagId );

		data = this.makeCall( args );
    
		return data;
	}
                    
	                
	/**
	 * Get a listing of tags
	 *
	 * 
	 *
	 	 * @param HashMap<String, String> optionalParameters A struct of optional parameters, see below for valid keys
	 * @return Object Array of records with the key of tagId and values of tagId, name
	 */ 
	public Object Tag_List( HashMap<String, String> optionalParameters ) {
		Object data;
		List <BasicNameValuePair> args = new ArrayList <BasicNameValuePair>();
		args = this.makeArg( args, "method", "Tag_List" );
		args = this.makeArg( args, "optionalParameters", optionalParameters );

		data = this.makeCall( args );
    
		return data;
	}
    
	
	public List <BasicNameValuePair> makeArg( List <BasicNameValuePair> args, String name, String value ) {
		args.add( new BasicNameValuePair( name, value ) );
		return args;
	}
	
	public List <BasicNameValuePair> makeArg( List <BasicNameValuePair> args, String name, int value ) {
		args.add( new BasicNameValuePair( name, Integer.toString( value ) ) );
		return args;
	}
	
	public List <BasicNameValuePair> makeArg( List <BasicNameValuePair> args, String name, NameValuePair value ) {
		args.add( new BasicNameValuePair( name+"["+value.getName()+"]", value.getValue() ) );
		return args;
	}
	
	public List <BasicNameValuePair> makeArg( List <BasicNameValuePair> args, String name, ArrayList<?> value ) {
		for( int i = 0; i < value.size(); i++ ) {
			Object o = value.get( i );
			if ( o instanceof HashMap ) {
				HashMap<?,?> h = (HashMap<?,?>) value.get( i );
				this.makeArg( args, name+"["+Integer.toString(i)+"]", h );
			} else if ( o instanceof BasicNameValuePair ) {
				BasicNameValuePair b = (BasicNameValuePair) value.get( i );
				this.makeArg( args, name, b );
			} else if ( o instanceof ArrayList ) {
				ArrayList<?> a = (ArrayList<?>) value.get( i );
				this.makeArg( args, name, a );
			} else {
				this.makeArg( args, name+"["+Integer.toString(i)+"]", o );
			}
		}
		return args;
	}
	
	public List <BasicNameValuePair> makeArg( List <BasicNameValuePair> args, String name, HashMap<?,?> value ) {
		for( Object key : value.keySet() ) {
			Object o = value.get( key );
			if ( o instanceof HashMap ) {
				HashMap<?,?> h = (HashMap<?,?>) value.get( key );
				this.makeArg( args, name+"["+key.toString()+"]", h );
			} else if ( o instanceof BasicNameValuePair ) {
				BasicNameValuePair b = (BasicNameValuePair) value.get( key );
				this.makeArg( args, name+"["+key.toString()+"]", b );
			} else if ( o instanceof ArrayList ) {
				ArrayList<?> a = (ArrayList<?>) value.get( key );
				this.makeArg( args, name+"["+key.toString()+"]", a );
			} else {
				this.makeArg( args, name+"["+key.toString()+"]", value.get( key ) );
			}
		}
		return args;
	}
	
	public List <BasicNameValuePair> makeArg( List <BasicNameValuePair> args, String name, Object value ) {
		args.add( new BasicNameValuePair( name, value.toString() ) );
		return args;
	}
	
	public Object makeCall( List <BasicNameValuePair> args ) {
		args.add( new BasicNameValuePair( "key", this.key ) );
		
		HttpClient httpclient = new DefaultHttpClient();

        HttpPost httppost = new HttpPost( this.url );
        httppost.addHeader( "Content-type", "application/x-www-form-urlencoded" );
        httppost.addHeader( "User-Agent", "SurveyTown Java Wrapper "+this.version );
        
        try {
        	httppost.setEntity( new UrlEncodedFormEntity(args, HTTP.UTF_8) );
        	
        	try { 		
	            HttpResponse response = httpclient.execute(httppost);
	            HttpEntity entity = response.getEntity();
	            
	            InputStream in = entity.getContent();
	            
	            StringBuilder sb = new StringBuilder();
	            String line;

	            try {
	                BufferedReader reader = new BufferedReader(new InputStreamReader(in, "UTF-8"));
	                while ((line = reader.readLine()) != null) {
	                    sb.append(line);
	                }
	            } finally {
	                in.close();
	            }
	            
	            Object o = JSONValue.parse( sb.toString() );
	            
	            return o;
	            
		    } catch (ClientProtocolException e) {
		            // TODO Auto-generated catch block
		            e.printStackTrace();
		    } catch (IOException e) {
		            // TODO Auto-generated catch block
		            e.printStackTrace();
		    }
        } catch ( UnsupportedEncodingException e ) {
        	e.printStackTrace();
        }
        
		return null;
	}
}
