<?php

class SurveyTown {
	protected $url = "api.myfeedback.io/2/REST/";
	protected $key;
	private $version = "2.5.0";
	
	/**
	 * Create a new SurveyTown object to interact with your SurveyTown account
	 *
	 * @param string key Your API Key - used to authenticate your account
	 */
	public function __construct( $key, $useHTTPS = false ) {
		$this->key = $key;
		$proto = "http://";
		if ( $useHTTPS )
			$proto = "https://";
		$this->url = "{$proto}{$this->url}";
	}
	
	                
	                
	/**
	 * Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE
	 *
	 * Please contact SurveyTown support for details, this function is only available to reseller accounts
	 *
	 	 * @param int clientId The client ID you are creating the cookie for, must be a client belonging to your reseller account
	 * @param string username The belonging to the client ID you are logging in with the cookie
	 * @param int time Cookie timeout in seconds
	 * @return string Cookie value to be set in SURVEYS_SSO_COOKIE
	 */ 
	public function Integration_Get_Cookie( $clientId, $username, $time = null ) {
		$args = array( 'key' => $this->key, 'method' => 'Integration_Get_Cookie' );
		$args['clientId'] = $clientId;
		$args['username'] = $username;
		$args['time'] = $time;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE using your client's credentials
	 *
	 * Please contact SurveyTown support for details, this function is only available to reseller accounts
	 *
	 	 * @param string username The of the client
	 * @param string password The current of the client
	 * @param int time Cookie timeout in seconds
	 * @return struct Returns a struct on a valid username/password combo
	 */ 
	public function Integration_Login_Get_Cookie( $username, $password, $time = null ) {
		$args = array( 'key' => $this->key, 'method' => 'Integration_Login_Get_Cookie' );
		$args['username'] = $username;
		$args['password'] = $password;
		$args['time'] = $time;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Retrieve a survey response
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to retrieve the response from
	 * @param int responseId The ID of the response you want to retrieve
	 * @return struct Array of response properties
	 */ 
	public function Response_Get( $surveyId, $responseId ) {
		$args = array( 'key' => $this->key, 'method' => 'Response_Get' );
		$args['surveyId'] = $surveyId;
		$args['responseId'] = $responseId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Create a survey response
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to respond to
	 * @param struct updateParameters A struct of update parameters, any combination of which can be used
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return int Returns responseId on success
	 */ 
	public function Response_Create( $surveyId, $updateParameters, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Response_Create' );
		$args['surveyId'] = $surveyId;
		$args['updateParameters'] = $updateParameters;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Update an existing survey response
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to update the response of
	 * @param int responseId The ID of the response you want to update
	 * @param struct updateParameters A struct of update parameters, any combination of which can be used
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return int Returns responseId on success
	 */ 
	public function Response_Update( $surveyId, $responseId, $updateParameters, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Response_Update' );
		$args['surveyId'] = $surveyId;
		$args['responseId'] = $responseId;
		$args['updateParameters'] = $updateParameters;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Delete a response from a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to delete from
	 * @param int responseId The ID of the response you want to delete
	 * @return bool Returns true on success
	 */ 
	public function Response_Delete( $surveyId, $responseId ) {
		$args = array( 'key' => $this->key, 'method' => 'Response_Delete' );
		$args['surveyId'] = $surveyId;
		$args['responseId'] = $responseId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Start a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to start
	 * @return bool Returns true on success
	 */ 
	public function Survey_Start( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Start' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Stop a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to stop
	 * @return bool Returns true on success
	 */ 
	public function Survey_Stop( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Stop' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Copy a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to copy
	 * @return int Returns The ID of the new survey
	 */ 
	public function Survey_Copy( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Copy' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Delete a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to delete
	 * @return bool Returns true on success
	 */ 
	public function Survey_Delete( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Delete' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Get a listing of surveys
	 *
	 * 
	 *
	 	 * @return struct Array of surveys with the key of surveyId
	 */ 
	public function Survey_List(  ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_List' );

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Find surveys based on specific criteria
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.
	 *
	 * 
	 * If the 'offset' optionalParameter is provided, the result set will start at that parameter's value (indexed from 0).
	 *
	 	 * @param struct optionalParameters A struct of optional parameters, any combination of which can be used
	 * @return struct Array of surveys with the key of surveyId and values of name, title, status, start_date, and end_date, ordered by survey ID ascending
	 */ 
	public function Survey_Find( $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Find' );
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Retrieve a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to retrieve
	 * @return struct Survey properties including questions, logic & actions
	 */ 
	public function Survey_Get_Info( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Get_Info' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Retrieves the questions for a particular survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want to retrieve the questions for
	 * @return struct Array of survey questions, with the key of the question ID. The response format is similar to that of Survey_Get_Responses.
	 */ 
	public function Survey_Get_Questions( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Get_Questions' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Counts how many responses a survey has.
	 *
	 * 
	 *
	 	 * @param int surveyId ID of the survey you want to retrieve the response counts for
	 * @return int Count of how many responses there are for the given survey
	 */ 
	public function Survey_Get_Response_Count( $surveyId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Get_Response_Count' );
		$args['surveyId'] = $surveyId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Retrieve responses a specific contact has entered
	 *
	 * 
	 *
	 	 * @param string emailAddress The email address of the contact you want to retrieve responses for
	 * @param struct optionalParameters A struct of optional parameters, any combination of which can be used
	 * @return struct Array of responses. See Survey_Get_Responses for response format documentation.
	 */ 
	public function Survey_Get_Responses_By_Email( $emailAddress, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Get_Responses_By_Email' );
		$args['emailAddress'] = $emailAddress;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Retrieve responses for a specific survey
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.
	 *
	 * 
	 * If the 'offset' optionalParameter is provided, the result set will start at that parameter's value (indexed from 0).
	 * 
	 * The return value is an array of responses. Each response has array keys for the question ID the response goes with.
	 * 
	 * The format for a simple textbox question's response looks like this example:
	 * 
	 * "24": {
	 *     "response": {
	 *         "choiceId": null,
	 *         "responseValue": "A Developer"
	 *     },
	 *     "questionText": "Who are you? - keep it to one line please.",
	 *     "questionId": 24,
	 *     "questionType": "singleText",
	 *     "token": null,
	 *     "isRequired": false
	 * }
	 * 
	 * Any question that can have only one response will have an array key named "response". Questions that can have more than one response value do /not/ have a "response" key; instead, they have a "responses" key, as seen in this example:
	 * 
	 * "29": {
	 *     "responses": [
	 *         {
	 *             "choiceId": 98,
	 *             "choiceText": "Blake",
	 *             "responseValue": 6
	 *         },
	 *         {
	 *             "choiceId": 99,
	 *             "choiceText": "Brian",
	 *             "responseValue": 5
	 *         },
	 *     ],
	 *     "questionText": "What number is each person thinking of? (remember, their numbers, and therefore also your guesses, must sum to 1001)",
	 *     "questionId": 29,
	 *     "questionType": "addSum",
	 *     "token": null,
	 *     "isRequired": false
	 * }
	 *
	 	 * @param int surveyId The ID of the survey you want to retrieve the questions for
	 * @param struct optionalParameters A struct of optional parameters, any combination of which can be used
	 * @return struct Array of surveys
	 */ 
	public function Survey_Get_Responses( $surveyId, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Get_Responses' );
		$args['surveyId'] = $surveyId;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Retrieve responses for a specific survey
	 *
	 * See documentation for Survey_Get_Responses for optionalParameters
	 *
	 	 */ 
	public function Survey_Get_Responses_Zapier(  ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Get_Responses_Zapier' );

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Add a tag to a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want the tag added to
	 * @param int tagId The ID of the tag you want to add
	 * @return bool Returns true on success
	 */ 
	public function Survey_Add_Tag( $surveyId, $tagId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Add_Tag' );
		$args['surveyId'] = $surveyId;
		$args['tagId'] = $tagId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Remove a tag from a survey
	 *
	 * 
	 *
	 	 * @param int surveyId The ID of the survey you want the tag removed from
	 * @param int tagId The ID of the tag you want to remove
	 * @return bool Returns true on success
	 */ 
	public function Survey_Remove_Tag( $surveyId, $tagId ) {
		$args = array( 'key' => $this->key, 'method' => 'Survey_Remove_Tag' );
		$args['surveyId'] = $surveyId;
		$args['tagId'] = $tagId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Create a new tag
	 *
	 * 
	 *
	 	 * @param string name The of your tag
	 * @return int id Returns the Tag ID of your new tag
	 */ 
	public function Tag_Create( $name ) {
		$args = array( 'key' => $this->key, 'method' => 'Tag_Create' );
		$args['name'] = $name;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Update an existing Tag
	 *
	 * Update Parameters can include:
	 *  - name: The name of your tag
	 *
	 	 * @param int tagId The of the tag you want to modify
	 * @param struct updateParameters A struct of replacement values for your tag - only specify items that you want to change
	 * @return bool Returns true on success
	 */ 
	public function Tag_Update( $tagId, $updateParameters ) {
		$args = array( 'key' => $this->key, 'method' => 'Tag_Update' );
		$args['tagId'] = $tagId;
		$args['updateParameters'] = $updateParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Delete a tag you created
	 *
	 * 
	 *
	 	 * @param int tagId The Tag ID of the tag you wish to delete
	 * @return bool Returns true on success
	 */ 
	public function Tag_Delete( $tagId ) {
		$args = array( 'key' => $this->key, 'method' => 'Tag_Delete' );
		$args['tagId'] = $tagId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Get a listing of tags
	 *
	 * 
	 *
	 	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return struct Array of records with the key of tagId and values of tagId, name
	 */ 
	public function Tag_List( $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Tag_List' );
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	    
	
	protected function makeCall( $args ) {
		$ch = curl_init( $this->url );
		curl_setopt($ch, CURLOPT_RETURNTRANSFER, true); 
		curl_setopt($ch, CURLOPT_POSTFIELDS, http_build_query( $args, null, "&" ) );
		curl_setopt($ch, CURLOPT_POST, true); 
		curl_setopt($ch, CURLOPT_USERAGENT, "SurveyTown PHP Wrapper {$this->version}" );
		$json = curl_exec( $ch );
		
		$data = json_decode( $json, true );
		
		if ( $data === false ) {
			throw new Exception( "Data could not be retrieved from API" );
		}
		if ( isset( $data['result'] ) && $data['result'] == "error" ) {
			throw new Exception( "API Error: {$data['message']}", $data['code'] );
		}

		return $data;
	}
}
