<?php
class SurveyTown_Admin extends SurveyTown {
	                
	                
	/**
	 * Create a new SurveyTown account with the given parameters
	 *
	 * 
	 *
	 	 * @param string clientName Name for the account
	 * @param string adminEmail Default email address for the account
	 * @param string userName User name for the new account - must be unique and no more than 64 characters
	 * @param string password Password for the new account
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return int The user ID of the new account
	 */ 
	public function Admin_Create_Account( $clientName, $adminEmail, $userName, $password, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Create_Account' );
		$args['clientName'] = $clientName;
		$args['adminEmail'] = $adminEmail;
		$args['userName'] = $userName;
		$args['password'] = $password;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Get a list of your current clients
	 *
	 * 
	 *
	 	 * @return struct Struct of current client accounts { clientId : clientName, clientId : clientName }
	 */ 
	public function Admin_Get_Accounts(  ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Get_Accounts' );

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Get a list of your clients based on parameters
	 *
	 * 
	 *
	 	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return struct Struct of accounts { { "clientId":"1", "clientStatus":"active", "clientCreated":"2012-01-01 00:00:00" }, { "clientId":"2", "clientStatus":"disabled", "clientCreated":"2012-02-02 02:00:00" }
	 */ 
	public function Admin_Find_Accounts( $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Find_Accounts' );
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Modify an existing SurveyTown account with the given parameters
	 *
	 * 
	 *
	 	 * @param int clientId The client ID number you want to modify
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return int The number of fields successfully modified
	 */ 
	public function Admin_Update_Account( $clientId, $optionalParameters ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Update_Account' );
		$args['clientId'] = $clientId;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Suspend an Account you created
	 *
	 * 
	 *
	 	 * @param int clientId The client ID number you want to modify
	 * @return bool true on success
	 */ 
	public function Admin_Suspend_Account( $clientId ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Suspend_Account' );
		$args['clientId'] = $clientId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Reinstate an Account you suspended
	 *
	 * 
	 *
	 	 * @param int clientId The client ID number you want to modify
	 * @return bool true on success
	 */ 
	public function Admin_Reinstate_Account( $clientId ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Reinstate_Account' );
		$args['clientId'] = $clientId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Creates an API key for the specified account.
	 *
	 * 
	 *
	 	 * @param int clientId The account ID number for which you want the API key
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return string API Key
	 */ 
	public function Admin_Create_Account_Key( $clientId, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Create_Account_Key' );
		$args['clientId'] = $clientId;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Return the API key for an account, or create one if one doesn't already exist and the username is not specified.
	 *
	 * 
	 *
	 	 * @param int clientId The account ID number for which you want the API key
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 * @return string API Key
	 */ 
	public function Admin_Get_Account_Key( $clientId, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Get_Account_Key' );
		$args['clientId'] = $clientId;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Send an Alert Message to a set of your clients
	 *
	 * 
	 *
	 	 * @param array clientIds An array of Client IDs who will receive the message
	 * @param string shortMessage A short message, up to 255 characters
	 * @param struct optionalParameters A struct of optional parameters, see below for valid keys
	 */ 
	public function Admin_Send_Message( $clientIds, $shortMessage, $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Send_Message' );
		$args['clientIds'] = $clientIds;
		$args['shortMessage'] = $shortMessage;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Get the current Webhooks settings for the specified Account
	 *
	 * 
	 *
	 	 * @param int clientId The client ID number you want to modify
	 * @return struct A struct of Webhook data
	 */ 
	public function Admin_Get_Account_Webhooks( $clientId ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Get_Account_Webhooks' );
		$args['clientId'] = $clientId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Admin_Set_Account_Webhooks allows you to set the Webhooks information for the specified Account
	 *
	 * The Account's current settings will be overriden, so specify all the hooks and locations you want
	 * 
	 * Note: To receive WEBHOOK_BOUNCED or WEBHOOK_CAMPAIGN_CLICKED, you must have the System location active
	 *
	 	 * @param int clientId The client ID number you want to modify
	 * @param string url The URL to receive the Webhook
	 * @param string webhooksKey webhookKey The Webhook Key passed along with the data to verify the origin, this is important for security
	 * @param array hooks An array of events you would like to receive Webfor. Valid values are: WEBHOOK_SUBSCRIBE, WEBHOOK_UNSUBSCRIBE, WEBHOOK_GLOBAL_UNSUBSCRIBE, WEBHOOK_PROFILE_UPDATE, WEBHOOK_BOUNCED, WEBHOOK_EMAIL_CHANGED, WEBHOOK_CAMPAIGN_OPENED, WEBHOOK_CAMPAIGN_CLICKED, WEBHOOK_CAMPAIGN_SHARED, WEBHOOK_CAMPAIGN_SENDING_STARTED, WEBHOOK_CAMPAIGN_SENT, WEBHOOK_CAMPAIGN_SENT_TO_ADDITIONAL_RECIPIENT, WEBHOOK_REACTIVATED, WEBHOOK_LIST_CREATED, WEBHOOK_LIST_DELETED, WEBHOOK_LIST_CHANGED
	 * @param array locations An array of Locations a particular Hook can be fired from. Valid values are: ContactDirect, Webapp, WebappBulk, System, API
	 * @param array customFieldIds An array of Custom Field IDs (See: Custom_Field_Get_All) - if specified, the values of the specified fields will be included in the data payload for any Webhook sending contact data
	 * @param array optionalParameters optionalParameters
	 * @return struct A struct of Webhook data
	 */ 
	public function Admin_Set_Account_Webhooks( $clientId, $url, $webhooksKey, $hooks, $locations, $customFieldIds = array(), $optionalParameters = array() ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Set_Account_Webhooks' );
		$args['clientId'] = $clientId;
		$args['url'] = $url;
		$args['webhooksKey'] = $webhooksKey;
		$args['hooks'] = $hooks;
		$args['locations'] = $locations;
		$args['customFieldIds'] = $customFieldIds;
		$args['optionalParameters'] = $optionalParameters;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	                    
	                
	/**
	 * Deactivate all Webhooks for an account
	 *
	 * Note: Webhooks already queued for sending will still be sent
	 *
	 	 * @param int clientId The client ID number you want to modify
	 * @return bool True on success
	 */ 
	public function Admin_Deactivate_Account_Webhooks( $clientId ) {
		$args = array( 'key' => $this->key, 'method' => 'Admin_Deactivate_Account_Webhooks' );
		$args['clientId'] = $clientId;

		$data = $this->makeCall( $args );
	    
		return $data;
	}
	    
}
