#!/usr/bin/env python

import httplib
try:
	import simplejson as json
except ImportError:
	import json
from urllib import quote


class SurveyTown:
	def __init__(self, key, useHTTPS=False):
		self.key = key
		self.host = "api.myfeedback.io:80"
		self.path = "/2/REST/"
		self.version = "2.5.0"
		self.useHTTPS = useHTTPS
		if self.useHTTPS:
			self.host = "api.myfeedback.io:443"


	def Integration_Get_Cookie(self, clientId, username, time):
		'''
		Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE

		Required keyword arguments:

		clientId (int) - The client ID you are creating the cookie for, must be a client belonging to your reseller account
		username (string) - The belonging to the client ID you are logging in with the cookie
		time (int) - Cookie timeout in seconds

		Optional keyword arguments:


		Returns string - Cookie value to be set in SURVEYS_SSO_COOKIE
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Integration_Get_Cookie',
			'clientId': clientId,
			'username': username,
			'time': time,
		}

		data = self.makeCall(args)
		return data


	def Integration_Login_Get_Cookie(self, username, password, time):
		'''
		Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE using your client's credentials

		Required keyword arguments:

		username (string) - The of the client
		password (string) - The current of the client
		time (int) - Cookie timeout in seconds

		Optional keyword arguments:


		Returns struct - Returns a struct on a valid username/password combo
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Integration_Login_Get_Cookie',
			'username': username,
			'password': password,
			'time': time,
		}

		data = self.makeCall(args)
		return data


	def Contact_Get(self, email, optionalParameters=None, **kwargs):
		'''
		Get information on a single contact

		Required keyword arguments:

		email (string) - The address of the contact you want information for
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		customFields (list) - Array of custom field IDs or tokens you want included with the results (ex: ['first_name','last_name','email_address'] OR [1,2,3])
		getAllCustomFields (bool) - As optionalParameter customFields, but automatically includes all custom fields.  Overrides customFields value

		Returns struct - Returns a struct of structs, keyed off of email address, each containing the keys specified above
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Contact_Get',
			'email': email,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Contact_Update(self, email, customFields, optionalParameters=None, **kwargs):
		'''
		Update the custom fields of an existing contact

		Required keyword arguments:

		email (string) - The address of the contact you want to update
		customFields (dict) - The custom fields you want to update and their new values
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		activateDeleted (bool) - If this contact was already deleted, reactivate them - defaults to true
		appendOptions (bool) - For use with checkbox lists, radio, and dropdown custom fields.  Append new values to the custom field definition dynamically
		checkboxListDelimiter (string) - For use with setting the values of checkbox list custom fields

		Returns bool - Returns true on success
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Contact_Update',
			'email': email,
			'customFields': customFields,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Contact_Change_Email(self, email, newEmail):
		'''
		Change the email address for an existing contact while preserving list and group subscriptions

		Required keyword arguments:

		email (string) - The current address of the contact
		newEmail (string) - The contact's new email address

		Optional keyword arguments:


		Returns struct - Returns a struct of structs, keyed off of email address, each containing the keys specified below
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Contact_Change_Email',
			'email': email,
			'newEmail': newEmail,
		}

		data = self.makeCall(args)
		return data


	def Contact_Activate(self, email):
		'''
		Return a contact to active status

		Required keyword arguments:

		email (string) - The address of the contact you wish to activate

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Contact_Activate',
			'email': email,
		}

		data = self.makeCall(args)
		return data


	def Contact_Delete(self, email):
		'''
		Delete a contact

		Required keyword arguments:

		email (string) - An address

		Optional keyword arguments:


		Returns bool - True if delete was successful
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Contact_Delete',
			'email': email,
		}

		data = self.makeCall(args)
		return data


	def Contact_Suppress(self, email, optionalParameters=None, **kwargs):
		'''
		Suppress a contact

		Required keyword arguments:

		email (string) - An address
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		emailId (int) - The ID of the email tied to this suppression

		Returns bool - Returns true or false indicating whether the contact was suppressed
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Contact_Suppress',
			'email': email,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Contact_Get_Person_Code(self, email):
		'''
		Retrieve the Person Code for a given contact

		Required keyword arguments:

		email (string) - The contact's address you want the Person Code for

		Optional keyword arguments:


		Returns string - Person Code
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Contact_Get_Person_Code',
			'email': email,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Textbox(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a Textbox CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Textbox',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_TextArea(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a TextArea CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_TextArea',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Password(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a Password CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Password',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Decimal(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a Decimal CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (float) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Decimal',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Currency(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a Currency CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (float) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Currency',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Integer(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add an Integer CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (int) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Integer',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Dropdown(self, fieldName, required, subscriberCanEdit, options, optionalParameters=None, **kwargs):
		'''
		Add a Dropdown CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		options (array) - An list of strings to be shown in the dropdown
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form - your value must be present in the options list

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Dropdown',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'options': options,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Radio(self, fieldName, required, subscriberCanEdit, options, optionalParameters=None, **kwargs):
		'''
		Add a Radio CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		options (array) - An list of strings, each value will have its own radio button
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you a radio button preselected on the form - your value must be present in the options list

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Radio',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'options': options,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Checkbox(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a single Checkbox CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (bool) - Set a defaultValue if you want this field checked automatically

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Checkbox',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_CheckboxList(self, fieldName, required, subscriberCanEdit, options, optionalParameters=None, **kwargs):
		'''
		Add a CheckboxList CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		options (array) - An list of strings, each value will have its own checkbox
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (list) - Each string in this list must correspond to a string in options, the checkbox for each string in defaultValue will be checked initially

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_CheckboxList',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'options': options,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Date(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a Date CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Date',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Email(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add an Email CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form - must be a valid email address

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Email',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Phone(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a Phone CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Phone',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_StateDropdown(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add a StateDropdown CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this value preselected in the dropdown - must be the full name of a valid state

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_StateDropdown',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Add_Address(self, fieldName, required, subscriberCanEdit, optionalParameters=None, **kwargs):
		'''
		Add an Address CustomField to your signup form

		Required keyword arguments:

		fieldName (string) - The name of your CustomField - this will be the label for your field on the form
		required (bool) - Is this field when the form is filled out?
		subscriberCanEdit (bool) - Can the subscriber edit this value later?
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		displayOrder (int) - A number indicating what order this field should appear in
		defaultValue (string) - Set a defaultValue if you want this field prepopulated on the form

		Returns struct - Returns a struct containing the new CustomFields fieldId and token
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'CustomField_Add_Address',
			'fieldName': fieldName,
			'required': required,
			'subscriberCanEdit': subscriberCanEdit,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Find(self, searchParameters=None):
		'''
		Get a list of CustomFields - excluding searchParameters indicates you want a list of all CustomFields

		Required keyword arguments:

		searchParameters (dict) - Provide a dict to narrow your results

		Optional keyword arguments:


		Returns struct - Returns a struct for each CustomField found
		'''
	
		if searchParameters is None:
			searchParameters = {}
	
		args = {
			'key': self.key,
			'method': 'CustomField_Find',
			'searchParameters': searchParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Get_All(self):
		'''
		Get a list of all current CustomFields

		Required keyword arguments:


		Optional keyword arguments:


		Returns struct - Returns a struct for each CustomField found
		'''
	
	
		args = {
			'key': self.key,
			'method': 'CustomField_Get_All',
		}

		data = self.makeCall(args)
		return data


	def CustomField_Update(self, fieldId, updateParameters):
		'''
		Update an existing CustomField

		Required keyword arguments:

		fieldId (int) - The of the CustomField you want to modify
		updateParameters (dict) - A dict of replacement values for your CustomField - only specify fields that you want to change

		Optional keyword arguments:


		Returns struct - Returns a struct with your CustomField's new properties
		'''
	
	
		args = {
			'key': self.key,
			'method': 'CustomField_Update',
			'fieldId': fieldId,
			'updateParameters': updateParameters,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Reorder(self, reorder):
		'''
		Update the displayOrder property on multiple fields at once

		Required keyword arguments:

		reorder (dict) - A dict where each key is a fieldId and each value is the new displayOrder

		Optional keyword arguments:


		Returns struct - Returns a struct for each CustomField updated
		'''
	
	
		args = {
			'key': self.key,
			'method': 'CustomField_Reorder',
			'reorder': reorder,
		}

		data = self.makeCall(args)
		return data


	def CustomField_Delete(self, fieldId):
		'''
		Delete a CustomField - this action cannot be undone

		Required keyword arguments:

		fieldId (int) - The of the CustomField you want to delete - first_name, last_name and email_address cannot be deleted

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'CustomField_Delete',
			'fieldId': fieldId,
		}

		data = self.makeCall(args)
		return data


	def Response_Get(self, surveyId, responseId):
		'''
		Retrieve a survey response

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve the response from
		responseId (int) - The ID of the response you want to retrieve

		Optional keyword arguments:


		Returns struct - Array of response properties
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Response_Get',
			'surveyId': surveyId,
			'responseId': responseId,
		}

		data = self.makeCall(args)
		return data


	def Response_Create(self, surveyId, updateParameters, optionalParameters=None, **kwargs):
		'''
		Create a survey response

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to respond to
		updateParameters (dict) - A dict of update parameters, any combination of which can be used
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		isComplete (bool) - Set a response as completed
		personCode (string) - A unique respondent identifier
		urlVariables (dict) - A dict of URL Variables: {name=>value}

		Returns int - Returns responseId on success
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Response_Create',
			'surveyId': surveyId,
			'updateParameters': updateParameters,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Response_Update(self, surveyId, responseId, updateParameters, optionalParameters=None, **kwargs):
		'''
		Update an existing survey response

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to update the response of
		responseId (int) - The ID of the response you want to update
		updateParameters (dict) - A dict of update parameters, any combination of which can be used
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		isComplete (bool) - Set a response as completed
		personCode (string) - A unique respondent identifier
		urlVariables (dict) - A dict of URL Variables: {name=>value}

		Returns int - Returns responseId on success
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Response_Update',
			'surveyId': surveyId,
			'responseId': responseId,
			'updateParameters': updateParameters,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Response_Delete(self, surveyId, responseId):
		'''
		Delete a response from a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to delete from
		responseId (int) - The ID of the response you want to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Response_Delete',
			'surveyId': surveyId,
			'responseId': responseId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Start(self, surveyId):
		'''
		Start a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to start

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Start',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Stop(self, surveyId):
		'''
		Stop a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to stop

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Stop',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Copy(self, surveyId):
		'''
		Copy a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to copy

		Optional keyword arguments:


		Returns int - Returns The ID of the new survey
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Copy',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Delete(self, surveyId):
		'''
		Delete a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Delete',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_List(self):
		'''
		Get a listing of surveys

		Required keyword arguments:


		Optional keyword arguments:


		Returns struct - Array of surveys with the key of surveyId
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_List',
		}

		data = self.makeCall(args)
		return data


	def Survey_Find(self, optionalParameters=None, **kwargs):
		'''
		Find surveys based on specific criteria
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.

		Required keyword arguments:

		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		surveyId (int) - Return only the survey with the supplied ID
		status (word) - Return only surveys matching this status. Must be one of "draft", "active", or "complete".
		type (word) - Return only surveys matching this type. Must be one of "survey" or "event".
		tagId (int) - Return only surveys having this tag ID
		startDate (date) - Return only surveys started on this date (UTC)
		endDate (date) - Return only surveys ending on this date (UTC)
		page (int) - Return surveys on this page of the search results
		offset (int) - Starting index to return results from. Indexed from 0.
		num (int) - Number of results to return

		Returns struct - Array of surveys with the key of surveyId and values of name, title, status, start_date, and end_date, ordered by survey ID ascending
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Find',
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Info(self, surveyId):
		'''
		Retrieve a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve

		Optional keyword arguments:


		Returns struct - Survey properties including questions, logic & actions
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Get_Info',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Questions(self, surveyId):
		'''
		Retrieves the questions for a particular survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve the questions for

		Optional keyword arguments:


		Returns struct - Array of survey questions, with the key of the question ID. The response format is similar to that of Survey_Get_Responses.
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Get_Questions',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Response_Count(self, surveyId):
		'''
		Counts how many responses a survey has.

		Required keyword arguments:

		surveyId (int) - ID of the survey you want to retrieve the response counts for

		Optional keyword arguments:


		Returns int - Count of how many responses there are for the given survey
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Get_Response_Count',
			'surveyId': surveyId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Responses_By_Email(self, emailAddress, optionalParameters=None, **kwargs):
		'''
		Retrieve responses a specific contact has entered

		Required keyword arguments:

		emailAddress (string) - The email address of the contact you want to retrieve responses for
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		surveyId (int) - Return only survey responses for this survey
		startDate (date) - Return only survey responses started on this date (UTC)
		endDate (date) - Return only survey responses completed on this date (UTC)
		is_complete (bool) - Return only survey responses that are complete (true) or incomplete (false)
		page (int) - Return questions on this page of the search results
		offset (int) - Starting index to return results from. Indexed from 0.
		num (int) - Number of results to return
		response_id (int) - Unique response identifier
		sortDir (string) - Sort direction, valid values: U, D

		Returns struct - Array of responses. See Survey_Get_Responses for response format documentation.
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Get_Responses_By_Email',
			'emailAddress': emailAddress,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Responses(self, surveyId, optionalParameters=None, **kwargs):
		'''
		Retrieve responses for a specific survey
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want to retrieve the questions for
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		startDate (date) - Return only survey responses started on this date (UTC)
		endDate (date) - Return only survey responses completed on this date (UTC)
		completeDate (date) - Return only survey responses completed on or after this date (UTC)
		is_complete (bool) - Return only survey responses that are complete (true) or incomplete (false)
		page (int) - Return questions on this page of the search results
		offset (int) - Starting index to return results from. Indexed from 0.
		num (int) - Number of results to return
		response_id (int) - Unique response identifier
		contact_id (int) - Unique respondent indentifier created
		person_code (string) - Unique respondent indentifier provided
		sortDir (string) - Sort direction, valid values: U, D. Defaults to D.

		Returns struct - Array of surveys
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Get_Responses',
			'surveyId': surveyId,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def Survey_Get_Responses_Zapier(self):
		'''
		Retrieve responses for a specific survey

		Required keyword arguments:


		Optional keyword arguments:


		Returns  - 
		'''
	
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Survey_Get_Responses_Zapier',
		}

		data = self.makeCall(args)
		return data


	def Survey_Add_Tag(self, surveyId, tagId):
		'''
		Add a tag to a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want the tag added to
		tagId (int) - The ID of the tag you want to add

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Add_Tag',
			'surveyId': surveyId,
			'tagId': tagId,
		}

		data = self.makeCall(args)
		return data


	def Survey_Remove_Tag(self, surveyId, tagId):
		'''
		Remove a tag from a survey

		Required keyword arguments:

		surveyId (int) - The ID of the survey you want the tag removed from
		tagId (int) - The ID of the tag you want to remove

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Survey_Remove_Tag',
			'surveyId': surveyId,
			'tagId': tagId,
		}

		data = self.makeCall(args)
		return data


	def Tag_Create(self, name):
		'''
		Create a new tag

		Required keyword arguments:

		name (string) - The of your tag

		Optional keyword arguments:


		Returns int - id Returns the Tag ID of your new tag
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Tag_Create',
			'name': name,
		}

		data = self.makeCall(args)
		return data


	def Tag_Update(self, tagId, updateParameters):
		'''
		Update an existing Tag

		Required keyword arguments:

		tagId (int) - The of the tag you want to modify
		updateParameters (dict) - A dict of replacement values for your tag - only specify items that you want to change

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Tag_Update',
			'tagId': tagId,
			'updateParameters': updateParameters,
		}

		data = self.makeCall(args)
		return data


	def Tag_Delete(self, tagId):
		'''
		Delete a tag you created

		Required keyword arguments:

		tagId (int) - The Tag ID of the tag you wish to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'Tag_Delete',
			'tagId': tagId,
		}

		data = self.makeCall(args)
		return data


	def Tag_List(self, optionalParameters=None, **kwargs):
		'''
		Get a listing of tags

		Required keyword arguments:

		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		sortBy (string) - Field to sort by, value values: name, id
		sortDir (string) - Sort direction, valid values: U, D
		offset (int) - The number of records to skip (specify a starting point)
		num (int) - The maximum number of records to return

		Returns struct - Array of records with the key of tagId and values of tagId, name
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'Tag_List',
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Create(self, surveyId, subject, content, optionalParameters=None, **kwargs):
		'''
		Create email message

		Required keyword arguments:

		surveyId (int) - The survey to use for the Email
		subject (string) - The line of the Email
		content (dict) - A dict which specifies the of the Email - can include html and text
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		senderName (string) - The from name of the Email
		senderEmail (string) - The from email address of the Email
		replyToName (string) - The display name of the replyToEmail
		replyToEmail (string) - The email address that replies to your Campaign should go to
		reminderInterval (string) - The number of days to wait before sending the email again. Only sent if the recipient does not open the original email.
		recurringInterval (string) - The number of days to wait before sending the email again. Sent regardless of prior responses. 

		Returns mixed - Returns emailId on success, returns a struct on failure
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'EmailMessage_Create',
			'surveyId': surveyId,
			'subject': subject,
			'content': content,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Update(self, emailId, optionalParameters=None, **kwargs):
		'''
		Update email message

		Required keyword arguments:

		emailId (int) - The Email ID
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		subject (string) - The subject line of the Email
		content (dict) - A dict which specifies the content of the Email - can include html and text
		senderName (string) - The from name of the Email
		senderEmail (string) - The from email address of the Email
		replyToName (string) - The display name of the replyToEmail
		replyToEmail (string) - The email address that replies to your Campaign should go to
		reminderInterval (string) - The number of days to wait before sending the email again. Only sent if the recipient does not open the original email.
		recurringInterval (string) - The number of months to wait before sending the email again. Sent regardless of prior responses.

		Returns mixed - Returns true on success, returns a struct on failure
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'EmailMessage_Update',
			'emailId': emailId,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Test(self, emailId, contacts, optionalParameters=None, **kwargs):
		'''
		Send test email message

		Required keyword arguments:

		emailId (int) - The Email ID
		contacts (array) - An list of dicts containing the you wish to send this email to
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		subject (string) - The subject line of the Email
		content (dict) - A dict which specifies the content of the Email - can include html and text
		senderName (string) - The from name of the Email
		senderEmail (string) - The from email address of the Email
		replyToName (string) - The display name of the replyToEmail
		replyToEmail (string) - The email address that replies to your Campaign should go to

		Returns mixed - Returns true on success, returns a struct on failure
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'EmailMessage_Test',
			'emailId': emailId,
			'contacts': contacts,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Delete(self, emailId):
		'''
		Delete a specified email

		Required keyword arguments:

		emailId (int) - The email you wish to delete

		Optional keyword arguments:


		Returns bool - Returns true on success
		'''
	
	
		args = {
			'key': self.key,
			'method': 'EmailMessage_Delete',
			'emailId': emailId,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Subscribe(self, emailId, contacts, optionalParameters=None, **kwargs):
		'''
		Send a contact an email message. No message is sent if the contact is already subscribed, use EmailMessage_Send instead.

		Required keyword arguments:

		emailId (int) - The Email ID
		contacts (array) - An list of dicts containing the you wish to send this email to
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		sendAt (datetime) -  Send the message around a specific datetime (UTC). Will send at the next oppourunity by default.

		Returns mixed - Returns true on success, returns a struct on failure
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'EmailMessage_Subscribe',
			'emailId': emailId,
			'contacts': contacts,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Send(self, emailId, contacts, optionalParameters=None, **kwargs):
		'''
		Send a contact an email message

		Required keyword arguments:

		emailId (int) - The Email ID
		contacts (array) - An list of dicts containing the you wish to send this email to
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:

		sendAt (datetime) -  Send the message around a specific datetime (UTC). Will send at the next oppourunity by default.

		Returns mixed - Returns true on success, returns a struct on failure
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'EmailMessage_Send',
			'emailId': emailId,
			'contacts': contacts,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Unsubscribe(self, emailId, emails):
		'''
		Unsubscribe a contact from an email message

		Required keyword arguments:

		emailId (int) - The Email ID
		emails (array) - An list of email addresses you are unsubscribing

		Optional keyword arguments:


		Returns mixed - Returns true on success, returns a struct on failure
		'''
	
	
		args = {
			'key': self.key,
			'method': 'EmailMessage_Unsubscribe',
			'emailId': emailId,
			'emails': emails,
		}

		data = self.makeCall(args)
		return data


	def EmailMessage_Find(self, surveyId, optionalParameters=None, **kwargs):
		'''
		Find an existing email message

		Required keyword arguments:

		surveyId (int) - The survey to use for the Email
		optionalParameters (dict) - Deprecated- use keyword arguments instead

		Optional keyword arguments:


		Returns mixed - Returns struct
		'''
	
		if optionalParameters is None:
			optionalParameters = {}
	
		for k,v in kwargs.iteritems():
			optionalParameters[k] = v

		args = {
			'key': self.key,
			'method': 'EmailMessage_Find',
			'surveyId': surveyId,
			'optionalParameters': optionalParameters,
		}

		data = self.makeCall(args)
		return data



	def http_build_query(self, params, topkey=''):
		
		if len(params) == 0:
			return ""
		
		result = ""
		
		if isinstance(params, dict):
			for key in params.keys():
				newkey = quote(key)
				if topkey != '':
					newkey = topkey + quote('[' + key + ']')
			
				if isinstance(params[key], dict):
					result += self.http_build_query(params[key], newkey)
			
				elif isinstance(params[key], list):
					i = 0
					for val in params[key]:
						if isinstance(val, dict):
							for subkey in val.keys():
								val[subkey] = str(val[subkey]).replace("'", "EC_SingleQuoteSentinelValue")
						result += newkey + quote('[' + str(i) + ']') + "=" + quote(str(val).replace('"', r'\"').replace("'",'"')) + "&"
						i = i + 1
			
				# boolean should have special treatment as well
				elif isinstance(params[key], bool):
					result += newkey + "=" + quote(str(int(params[key]))) + "&"
			
				# assume string (integers and floats work well)
				else:
					result += newkey + "=" + quote(str(params[key]).replace( "'", "EC_SingleQuoteSentinelValue" )) + "&"
		
		# remove the last '&'
		if result and topkey == '' and result[-1] == '&':
			result = result[:-1]
			
		result = result.replace("EC_SingleQuoteSentinelValue", "'")
		
		return result
	

	def makeCall(self, args):
		params = self.http_build_query(args)
		headers = {"Content-type": "application/x-www-form-urlencoded", "Accept": "text/plain", "User-Agent": "SurveyTown Python Wrapper " + self.version}
		
		if self.useHTTPS:
			conn = httplib.HTTPSConnection(self.host)
		else:
			conn = httplib.HTTPConnection(self.host)
		conn.request("POST", self.path, params, headers)
		
		response = conn.getresponse()
		jsonstr = response.read()
		
		data = None
		data = json.loads(jsonstr)
		
		if data is None:
			raise Exception("Could not fetch data from the API.")

		if isinstance(data, dict) and 'result' in data and data['result'] == "error":
			raise Exception("API Error: "+data['message']+" ("+repr(data['code'])+")")
		
		return data

		
class SurveyTown_Reseller(SurveyTown):

