using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.Text;
using Newtonsoft.Json;

namespace SurveyTownREST {
	public class SurveyTown {
        private Transport transport;
		
        public SurveyTown(string api_key, bool use_https) {
        	transport = new Transport(api_key, use_https, "");
    	}

		    		
    	    		
    /**
     * <summary>Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE</summary>
     *
     * Please contact SurveyTown support for details, this function is only available to reseller accounts
     *
	 * <param>clientId The client ID you are creating the cookie for, must be a client belonging to your reseller account</param>
	 * <param>stringusername The belonging to the client ID you are logging in with the cookie</param>
	 * <param>time Cookie timeout in seconds</param>
	 * <returns>object Cookie value to be set in SURVEYS_SSO_COOKIE</returns>
	 */
    	public object Integration_Get_Cookie( int clientId, string username, int time ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);
			args.Add("username", username);
			args.Add("time", time);

    		return transport.MakeCall("Integration_Get_Cookie", args);
    	}
    	    		
    	    		
    /**
     * <summary>Integrate SurveyTown into your own web application by setting your SURVEYS_SSO_COOKIE using your client's credentials</summary>
     *
     * Please contact SurveyTown support for details, this function is only available to reseller accounts
     *
	 * <param>stringusername The of the client</param>
	 * <param>stringpassword The current of the client</param>
	 * <param>time Cookie timeout in seconds</param>
	 * <returns>object Returns a struct on a valid username/password combo</returns>
	 */
    	public object Integration_Login_Get_Cookie( string username, string password, int time ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("username", username);
			args.Add("password", password);
			args.Add("time", time);

    		return transport.MakeCall("Integration_Login_Get_Cookie", args);
    	}
    	    		
    	    		
    /**
     * <summary>Retrieve a survey response</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to retrieve the response from</param>
	 * <param>responseId The ID of the response you want to retrieve</param>
	 * <returns>object Array of response properties</returns>
	 */
    	public object Response_Get( int surveyId, int responseId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("responseId", responseId);

    		return transport.MakeCall("Response_Get", args);
    	}
    	    		
    	    		
    /**
     * <summary>Create a survey response</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to respond to</param>
	 * <param>Dictionary<string, string>updateParameters A struct of update parameters, any combination of which can be used</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object Returns responseId on success</returns>
	 */
    	public object Response_Create( int surveyId, Dictionary<string, string> updateParameters, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("updateParameters", updateParameters);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Response_Create", args);
    	}
    	    		
    	    		
    /**
     * <summary>Update an existing survey response</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to update the response of</param>
	 * <param>responseId The ID of the response you want to update</param>
	 * <param>Dictionary<string, string>updateParameters A struct of update parameters, any combination of which can be used</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object Returns responseId on success</returns>
	 */
    	public object Response_Update( int surveyId, int responseId, Dictionary<string, string> updateParameters, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("responseId", responseId);
			args.Add("updateParameters", updateParameters);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Response_Update", args);
    	}
    	    		
    	    		
    /**
     * <summary>Delete a response from a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to delete from</param>
	 * <param>responseId The ID of the response you want to delete</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Response_Delete( int surveyId, int responseId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("responseId", responseId);

    		return transport.MakeCall("Response_Delete", args);
    	}
    	    		
    	    		
    /**
     * <summary>Start a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to start</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Survey_Start( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Start", args);
    	}
    	    		
    	    		
    /**
     * <summary>Stop a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to stop</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Survey_Stop( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Stop", args);
    	}
    	    		
    	    		
    /**
     * <summary>Copy a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to copy</param>
	 * <returns>object Returns The ID of the new survey</returns>
	 */
    	public object Survey_Copy( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Copy", args);
    	}
    	    		
    	    		
    /**
     * <summary>Delete a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to delete</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Survey_Delete( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Delete", args);
    	}
    	    		
    	    		
    /**
     * <summary>Get a listing of surveys</summary>
     *
     * 
     *
	 * <returns>object Array of surveys with the key of surveyId</returns>
	 */
    	public object Survey_List(  ) {
    		object data;
    		Arguments args = new Arguments();


    		return transport.MakeCall("Survey_List", args);
    	}
    	    		
    	    		
    /**
     * <summary>Find surveys based on specific criteria
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.</summary>
     *
     * 
	 * If the 'offset' optionalParameter is provided, the result set will start at that parameter's value (indexed from 0).
     *
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, any combination of which can be used</param>
	 * <returns>object Array of surveys with the key of surveyId and values of name, title, status, start_date, and end_date, ordered by survey ID ascending</returns>
	 */
    	public object Survey_Find( Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Survey_Find", args);
    	}
    	    		
    	    		
    /**
     * <summary>Retrieve a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to retrieve</param>
	 * <returns>object Survey properties including questions, logic & actions</returns>
	 */
    	public object Survey_Get_Info( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Get_Info", args);
    	}
    	    		
    	    		
    /**
     * <summary>Retrieves the questions for a particular survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want to retrieve the questions for</param>
	 * <returns>object Array of survey questions, with the key of the question ID. The response format is similar to that of Survey_Get_Responses.</returns>
	 */
    	public object Survey_Get_Questions( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Get_Questions", args);
    	}
    	    		
    	    		
    /**
     * <summary>Counts how many responses a survey has.</summary>
     *
     * 
     *
	 * <param>surveyId ID of the survey you want to retrieve the response counts for</param>
	 * <returns>object Count of how many responses there are for the given survey</returns>
	 */
    	public object Survey_Get_Response_Count( int surveyId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);

    		return transport.MakeCall("Survey_Get_Response_Count", args);
    	}
    	    		
    	    		
    /**
     * <summary>Retrieve responses a specific contact has entered</summary>
     *
     * 
     *
	 * <param>stringemailAddress The email address of the contact you want to retrieve responses for</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, any combination of which can be used</param>
	 * <returns>object Array of responses. See Survey_Get_Responses for response format documentation.</returns>
	 */
    	public object Survey_Get_Responses_By_Email( string emailAddress, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("emailAddress", emailAddress);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Survey_Get_Responses_By_Email", args);
    	}
    	    		
    	    		
    /**
     * <summary>Retrieve responses for a specific survey
If the 'page' optionalParameter is provided, results are paginated to 1000 surveys per page.</summary>
     *
     * 
	 * If the 'offset' optionalParameter is provided, the result set will start at that parameter's value (indexed from 0).
	 * 
	 * The return value is an array of responses. Each response has array keys for the question ID the response goes with.
	 * 
	 * The format for a simple textbox question's response looks like this example:
	 * 
	 * "24": {
	 *     "response": {
	 *         "choiceId": null,
	 *         "responseValue": "A Developer"
	 *     },
	 *     "questionText": "Who are you? - keep it to one line please.",
	 *     "questionId": 24,
	 *     "questionType": "singleText",
	 *     "token": null,
	 *     "isRequired": false
	 * }
	 * 
	 * Any question that can have only one response will have an array key named "response". Questions that can have more than one response value do /not/ have a "response" key; instead, they have a "responses" key, as seen in this example:
	 * 
	 * "29": {
	 *     "responses": [
	 *         {
	 *             "choiceId": 98,
	 *             "choiceText": "Blake",
	 *             "responseValue": 6
	 *         },
	 *         {
	 *             "choiceId": 99,
	 *             "choiceText": "Brian",
	 *             "responseValue": 5
	 *         },
	 *     ],
	 *     "questionText": "What number is each person thinking of? (remember, their numbers, and therefore also your guesses, must sum to 1001)",
	 *     "questionId": 29,
	 *     "questionType": "addSum",
	 *     "token": null,
	 *     "isRequired": false
	 * }
     *
	 * <param>surveyId The ID of the survey you want to retrieve the questions for</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, any combination of which can be used</param>
	 * <returns>object Array of surveys</returns>
	 */
    	public object Survey_Get_Responses( int surveyId, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Survey_Get_Responses", args);
    	}
    	    		
    	    		
    /**
     * <summary>Retrieve responses for a specific survey</summary>
     *
     * See documentation for Survey_Get_Responses for optionalParameters
     *
	 */
    	public object Survey_Get_Responses_Zapier(  ) {
    		object data;
    		Arguments args = new Arguments();


    		return transport.MakeCall("Survey_Get_Responses_Zapier", args);
    	}
    	    		
    	    		
    /**
     * <summary>Add a tag to a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want the tag added to</param>
	 * <param>tagId The ID of the tag you want to add</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Survey_Add_Tag( int surveyId, int tagId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("tagId", tagId);

    		return transport.MakeCall("Survey_Add_Tag", args);
    	}
    	    		
    	    		
    /**
     * <summary>Remove a tag from a survey</summary>
     *
     * 
     *
	 * <param>surveyId The ID of the survey you want the tag removed from</param>
	 * <param>tagId The ID of the tag you want to remove</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Survey_Remove_Tag( int surveyId, int tagId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("surveyId", surveyId);
			args.Add("tagId", tagId);

    		return transport.MakeCall("Survey_Remove_Tag", args);
    	}
    	    		
    	    		
    /**
     * <summary>Create a new tag</summary>
     *
     * 
     *
	 * <param>stringname The of your tag</param>
	 * <returns>object id Returns the Tag ID of your new tag</returns>
	 */
    	public object Tag_Create( string name ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("name", name);

    		return transport.MakeCall("Tag_Create", args);
    	}
    	    		
    	    		
    /**
     * <summary>Update an existing Tag</summary>
     *
     * Update Parameters can include:
	 *  - name: The name of your tag
     *
	 * <param>tagId The of the tag you want to modify</param>
	 * <param>Dictionary<string, string>updateParameters A struct of replacement values for your tag - only specify items that you want to change</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Tag_Update( int tagId, Dictionary<string, string> updateParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("tagId", tagId);
			args.Add("updateParameters", updateParameters);

    		return transport.MakeCall("Tag_Update", args);
    	}
    	    		
    	    		
    /**
     * <summary>Delete a tag you created</summary>
     *
     * 
     *
	 * <param>tagId The Tag ID of the tag you wish to delete</param>
	 * <returns>object Returns true on success</returns>
	 */
    	public object Tag_Delete( int tagId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("tagId", tagId);

    		return transport.MakeCall("Tag_Delete", args);
    	}
    	    		
    	    		
    /**
     * <summary>Get a listing of tags</summary>
     *
     * 
     *
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object Array of records with the key of tagId and values of tagId, name</returns>
	 */
    	public object Tag_List( Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Tag_List", args);
    	}
    	
	}
}