using System;
using System.Collections.Generic;
using System.Collections.Specialized;
using System.Linq;
using System.Net;
using System.Text;
using Newtonsoft.Json;

namespace SurveyTownREST {
	public class SurveyTownReseller {
        private Transport transport;
		
        public SurveyTownReseller(string api_key, bool use_https) {
        	transport = new Transport(api_key, use_https, "");
    	}

		    		
    	    		
    /**
     * <summary>Create a new SurveyTown account with the given parameters</summary>
     *
     * 
     *
	 * <param>stringclientName Name for the account</param>
	 * <param>stringadminEmail Default email address for the account</param>
	 * <param>stringuserName User name for the new account - must be unique and no more than 64 characters</param>
	 * <param>stringpassword Password for the new account</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object The user ID of the new account</returns>
	 */
    	public object Admin_Create_Account( string clientName, string adminEmail, string userName, string password, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientName", clientName);
			args.Add("adminEmail", adminEmail);
			args.Add("userName", userName);
			args.Add("password", password);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Create_Account", args);
    	}
    	    		
    	    		
    /**
     * <summary>Get a list of your current clients</summary>
     *
     * 
     *
	 * <returns>object Struct of current client accounts { clientId : clientName, clientId : clientName }</returns>
	 */
    	public object Admin_Get_Accounts(  ) {
    		object data;
    		Arguments args = new Arguments();


    		return transport.MakeCall("Admin_Get_Accounts", args);
    	}
    	    		
    	    		
    /**
     * <summary>Get a list of your clients based on parameters</summary>
     *
     * 
     *
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object Struct of accounts { { "clientId":"1", "clientStatus":"active", "clientCreated":"2012-01-01 00:00:00" }, { "clientId":"2", "clientStatus":"disabled", "clientCreated":"2012-02-02 02:00:00" }</returns>
	 */
    	public object Admin_Find_Accounts( Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Find_Accounts", args);
    	}
    	    		
    	    		
    /**
     * <summary>Modify an existing SurveyTown account with the given parameters</summary>
     *
     * 
     *
	 * <param>clientId The client ID number you want to modify</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object The number of fields successfully modified</returns>
	 */
    	public object Admin_Update_Account( int clientId, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Update_Account", args);
    	}
    	    		
    	    		
    /**
     * <summary>Suspend an Account you created</summary>
     *
     * 
     *
	 * <param>clientId The client ID number you want to modify</param>
	 * <returns>object true on success</returns>
	 */
    	public object Admin_Suspend_Account( int clientId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);

    		return transport.MakeCall("Admin_Suspend_Account", args);
    	}
    	    		
    	    		
    /**
     * <summary>Reinstate an Account you suspended</summary>
     *
     * 
     *
	 * <param>clientId The client ID number you want to modify</param>
	 * <returns>object true on success</returns>
	 */
    	public object Admin_Reinstate_Account( int clientId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);

    		return transport.MakeCall("Admin_Reinstate_Account", args);
    	}
    	    		
    	    		
    /**
     * <summary>Creates an API key for the specified account.</summary>
     *
     * 
     *
	 * <param>clientId The account ID number for which you want the API key</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object API Key</returns>
	 */
    	public object Admin_Create_Account_Key( int clientId, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Create_Account_Key", args);
    	}
    	    		
    	    		
    /**
     * <summary>Return the API key for an account, or create one if one doesn't already exist and the username is not specified.</summary>
     *
     * 
     *
	 * <param>clientId The account ID number for which you want the API key</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 * <returns>object API Key</returns>
	 */
    	public object Admin_Get_Account_Key( int clientId, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Get_Account_Key", args);
    	}
    	    		
    	    		
    /**
     * <summary>Send an Alert Message to a set of your clients</summary>
     *
     * 
     *
	 * <param>List<object>clientIds An array of Client IDs who will receive the message</param>
	 * <param>stringshortMessage A short message, up to 255 characters</param>
	 * <param>Dictionary<string, string>optionalParameters A struct of optional parameters, see below for valid keys</param>
	 */
    	public object Admin_Send_Message( List<object> clientIds, string shortMessage, Dictionary<string, string> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientIds", clientIds);
			args.Add("shortMessage", shortMessage);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Send_Message", args);
    	}
    	    		
    	    		
    /**
     * <summary>Get the current Webhooks settings for the specified Account</summary>
     *
     * 
     *
	 * <param>clientId The client ID number you want to modify</param>
	 * <returns>object A struct of Webhook data</returns>
	 */
    	public object Admin_Get_Account_Webhooks( int clientId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);

    		return transport.MakeCall("Admin_Get_Account_Webhooks", args);
    	}
    	    		
    	    		
    /**
     * <summary>Admin_Set_Account_Webhooks allows you to set the Webhooks information for the specified Account</summary>
     *
     * The Account's current settings will be overriden, so specify all the hooks and locations you want
	 * 
	 * Note: To receive WEBHOOK_BOUNCED or WEBHOOK_CAMPAIGN_CLICKED, you must have the System location active
     *
	 * <param>clientId The client ID number you want to modify</param>
	 * <param>stringurl The URL to receive the Webhook</param>
	 * <param>stringwebhooksKey webhookKey The Webhook Key passed along with the data to verify the origin, this is important for security</param>
	 * <param>List<object>hooks An array of events you would like to receive Webfor. Valid values are: WEBHOOK_SUBSCRIBE, WEBHOOK_UNSUBSCRIBE, WEBHOOK_GLOBAL_UNSUBSCRIBE, WEBHOOK_PROFILE_UPDATE, WEBHOOK_BOUNCED, WEBHOOK_EMAIL_CHANGED, WEBHOOK_CAMPAIGN_OPENED, WEBHOOK_CAMPAIGN_CLICKED, WEBHOOK_CAMPAIGN_SHARED, WEBHOOK_CAMPAIGN_SENDING_STARTED, WEBHOOK_CAMPAIGN_SENT, WEBHOOK_CAMPAIGN_SENT_TO_ADDITIONAL_RECIPIENT, WEBHOOK_REACTIVATED, WEBHOOK_LIST_CREATED, WEBHOOK_LIST_DELETED, WEBHOOK_LIST_CHANGED</param>
	 * <param>List<object>locations An array of Locations a particular Hook can be fired from. Valid values are: ContactDirect, Webapp, WebappBulk, System, API</param>
	 * <param>List<object>customFieldIds An array of Custom Field IDs (See: Custom_Field_Get_All) - if specified, the values of the specified fields will be included in the data payload for any Webhook sending contact data</param>
	 * <param>List<object>optionalParameters optionalParameters</param>
	 * <returns>object A struct of Webhook data</returns>
	 */
    	public object Admin_Set_Account_Webhooks( int clientId, string url, string webhooksKey, List<object> hooks, List<object> locations, List<object> customFieldIds, List<object> optionalParameters ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);
			args.Add("url", url);
			args.Add("webhooksKey", webhooksKey);
			args.Add("hooks", hooks);
			args.Add("locations", locations);
			args.Add("customFieldIds", customFieldIds);
			args.Add("optionalParameters", optionalParameters);

    		return transport.MakeCall("Admin_Set_Account_Webhooks", args);
    	}
    	    		
    	    		
    /**
     * <summary>Deactivate all Webhooks for an account</summary>
     *
     * Note: Webhooks already queued for sending will still be sent
     *
	 * <param>clientId The client ID number you want to modify</param>
	 * <returns>object True on success</returns>
	 */
    	public object Admin_Deactivate_Account_Webhooks( int clientId ) {
    		object data;
    		Arguments args = new Arguments();

			args.Add("clientId", clientId);

    		return transport.MakeCall("Admin_Deactivate_Account_Webhooks", args);
    	}
    	
	}
}